# Copyright (c) 2015, Truelite Srl <info@truelite.it>
# Author: Mark Caglienzi <mark@truelite.it>
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import datetime
import decimal
import logging
import optparse
import random
import sys
# TODO: User is not to be fixed, but it is AUTH_USER_MODEL
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
import example_project.models as epmodels
import django_pettycash.consts as dpc_consts
import django_pettycash.models as dpcmodels
import django_pettycash.utils as dpcutils


log = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Fill the database with random data'
    option_list = BaseCommand.option_list + (
        optparse.make_option("--quiet", action="store_true", default=None, help="Disable progress reporting"),
    )

    def handle(self, *names, **opts):
        FORMAT = "%(asctime)-15s %(levelname)s %(message)s"
        if opts["quiet"]:
            logging.basicConfig(level=logging.WARNING, stream=sys.stderr, format=FORMAT)
        else:
            logging.basicConfig(level=logging.INFO, stream=sys.stderr, format=FORMAT)

        # Create a bunch of random data to help initial test of django_pettycash
        print "Creating random data..."
        for i in range(random.randint(4, 7)):
            epmodels.Company.objects.create(name="Company %02d" % i)
        dpcmodels.Source.objects.create(
            name='Bank',
            description='Bank account',
        )
        dpcmodels.Source.objects.create(
            name='Cash',
            description='Cash',
        )
        for i in range(random.randint(3, 5)):
            dpcmodels.Tag.objects.create(name='Tag %c' % (i + 65))
        # Get date ranges
        today = datetime.date.today()
        (first_day_current_month, last_day_current_month) = dpcutils.monthrange(today)
        (first_day_previous_month, last_day_previous_month) = dpcutils.monthrange(
            first_day_current_month - datetime.timedelta(days=1)
        )
        companies = list(epmodels.Company.objects.all())
        sources = list(dpcmodels.Source.objects.all())
        tags = list(dpcmodels.Tag.objects.all())
        # TODO: Fix the model, using AUTH_USER_MODEL
        author = User.objects.all()[0]
        day = first_day_previous_month
        while day <= today:
            for i in range(random.randint(1, 3)):
                entry = dpcmodels.Entry.objects.create(
                    company=random.choice(companies),
                    source=random.choice(sources),
                    kind=random.choice([dpc_consts.ENTRY_KIND_INBOUND, dpc_consts.ENTRY_KIND_OUTBOUND]),
                    amount=decimal.Decimal('%d.%02d' % (random.randint(1, 150), random.randint(0, 99))),
                    reason="Random reason %d" % random.randint(1, 150),
                    date=day,
                    created_by=author,
                    modified_by=author,
                )
                entry_tags = random.sample(tags, random.randint(0, len(tags)))
                for t in entry_tags:
                    entry.tags.add(t)
            day += datetime.timedelta(days=random.randint(1, 3))
        # Create coherent MonthlyBalance objects
        for source in sources:
            # Previous month
            monthly_balance_previous_month = dpcmodels.MonthlyBalance.objects.create(
                source=source,
                date=first_day_previous_month,
                start=decimal.Decimal('%d.%02d' % (random.randint(10, 100), random.randint(0, 99))),
            )
            monthly_balance_previous_month.update_balance(
                start=monthly_balance_previous_month.start
            )
            # Current month
            monthly_balance_current_month = dpcmodels.MonthlyBalance.objects.create(
                source=source,
                date=first_day_current_month,
                start=monthly_balance_previous_month.end,
            )
            monthly_balance_current_month.update_balance()
        print "Done!"
